package im.zego.realtimetranslationdemo;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.media.AudioFormat;
import android.media.AudioManager;
import android.media.AudioTrack;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.provider.MediaStore;
import android.text.TextUtils;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContract;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AlertDialog;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.google.gson.Gson;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import im.zego.realtimetranslation.IZegoRealtimeTranslationHandler;
import im.zego.realtimetranslation.ZegoRTServiceProvider;
import im.zego.realtimetranslation.ZegoRealtimeTranslation;
import im.zego.realtimetranslation.ZegoSpeechLanguage;

/**
 * A simple {@link Fragment} subclass.
 * Use the {@link AudioFileFragment#newInstance} factory method to
 * create an instance of this fragment.
 */
public class AudioFileFragment extends Fragment {
    private static final String TAG = "AudioFileFragment";
    private Button startRTFileButton;
    private CheckBox enableTranslateCheckBox;
    private CheckBox enableIntermediaTranslateCheckBox;
    private Spinner pcmFileSelector;
    private ArrayAdapter<ZegoConstants.SpeechLanguage> pcmFileAdapter;
    public static final ZegoConstants.SpeechLanguage[] pcmFileList = new ZegoConstants.SpeechLanguage[] {
            new ZegoConstants.SpeechLanguage(ZegoSpeechLanguage.ar_IL, "阿拉伯语"),
            new ZegoConstants.SpeechLanguage(ZegoSpeechLanguage.en_US, "英语"),
            new ZegoConstants.SpeechLanguage(ZegoSpeechLanguage.fr_FR, "法语"),
            new ZegoConstants.SpeechLanguage(ZegoSpeechLanguage.ko_KR, "韩语"),
            new ZegoConstants.SpeechLanguage(ZegoSpeechLanguage.ms_MY, "马来语"),
            new ZegoConstants.SpeechLanguage(ZegoSpeechLanguage.vi_VN, "越南语"),
            new ZegoConstants.SpeechLanguage(ZegoSpeechLanguage.id_ID, "印尼语")
    };

    public static final int [] pcmFileIDList = new int[] {
            R.raw.arabic,
            R.raw.english,
            R.raw.french,
            R.raw.korean,
            R.raw.malay,
            R.raw.vi,
            R.raw.yini
    } ;

    private RecyclerView speechToTextRecyclerView;
    private RecyclerView translateTextRecyclerView;
    public TextAdapter speechToTextAdapter = new TextAdapter();
    public TextAdapter translateTextAdapter = new TextAdapter();
    private Spinner sttProviderSelector;
    private ArrayAdapter<String> sttProviderAdapter;
    private Spinner translateProviderSelector;
    private ArrayAdapter<String> translateProviderAdapter;
    private Spinner translateLanguageSelector;
    private ArrayAdapter<ZegoConstants.TranslationLanguage> googleTranslateLanguageAdapter;
    private ArrayAdapter<ZegoConstants.TranslationLanguage> iFlyTransLanguagesAdapter;
    private ArrayAdapter<ZegoConstants.TranslationLanguage> iLiveDataTransLanguagesAdapter;
    private LiveActivityViewModel viewModel;
    private ZegoRealtimeTranslation zegoRealtimeTranslation = ZegoRealtimeTranslation.getInstance();
    private Gson gson = new Gson();
    private Handler handler = new Handler(Looper.getMainLooper());
    private boolean isStartRecognizeFile = false;
    public AudioFileFragment() {
        // Required empty public constructor
    }

    /**
     * Use this factory method to create a new instance of
     * this fragment using the provided parameters.
     *
     * @return A new instance of fragment AudioFileFragment.
     */
    public static AudioFileFragment newInstance() {
        return new AudioFileFragment();
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        viewModel = new ViewModelProvider(requireActivity()).get(LiveActivityViewModel.class);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View containerView = inflater.inflate(R.layout.fragment_audio_file, container, false);
        pcmFileSelector = containerView.findViewById(R.id.pcm_file_selector);
        startRTFileButton = containerView.findViewById(R.id.start_recognize_file_btn);
        speechToTextRecyclerView = containerView.findViewById(R.id.speech_to_text_recycler_view);
        translateTextRecyclerView = containerView.findViewById(R.id.translate_text_recycler_view);
        sttProviderSelector = containerView.findViewById(R.id.speech_to_text_sdk);
        translateProviderSelector = containerView.findViewById(R.id.translate_sdk);
        translateLanguageSelector = containerView.findViewById(R.id.to_language);
        enableTranslateCheckBox = containerView.findViewById(R.id.checkbox_enable_translate);
        enableIntermediaTranslateCheckBox = containerView.findViewById(R.id.checkbox_enable_intermedia_translate);
        containerView.findViewById(R.id.clear_screen).setOnClickListener(v -> {
            speechToTextAdapter.clear();
            translateTextAdapter.clear();
        });

        enableTranslateCheckBox.setOnCheckedChangeListener((buttonView, isChecked) -> {
            zegoRealtimeTranslation.enableTranslate(isChecked);
            if (isChecked) {
                enableIntermediaTranslateCheckBox.setVisibility(View.VISIBLE);
            } else {
                enableIntermediaTranslateCheckBox.setVisibility(View.GONE);
            }
        });
        enableIntermediaTranslateCheckBox.setOnCheckedChangeListener((buttonView, isChecked) -> {
            zegoRealtimeTranslation.enableInterimTranslation(isChecked);
        });

        LinearLayoutManager linearLayoutManager1 = new LinearLayoutManager(requireContext());
        linearLayoutManager1.setStackFromEnd(true);
        speechToTextRecyclerView.setLayoutManager(linearLayoutManager1);
        speechToTextRecyclerView.setAdapter(speechToTextAdapter);
        speechToTextAdapter.setRecyclerView(speechToTextRecyclerView);

        LinearLayoutManager linearLayoutManager2 = new LinearLayoutManager(requireContext());
        linearLayoutManager2.setStackFromEnd(true);
        translateTextRecyclerView.setLayoutManager(linearLayoutManager2);
        translateTextRecyclerView.setAdapter(translateTextAdapter);
        translateTextAdapter.setRecyclerView(translateTextRecyclerView);

        pcmFileAdapter = new ArrayAdapter<>(requireContext(), android.R.layout.simple_spinner_item, pcmFileList);
        pcmFileAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        pcmFileSelector.setAdapter(pcmFileAdapter);
        pcmFileSelector.setSelection(0);

        iFlyTransLanguagesAdapter = new ArrayAdapter<>(requireContext(), android.R.layout.simple_spinner_item, ZegoConstants.iflyTransLanguages);
        iFlyTransLanguagesAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        googleTranslateLanguageAdapter = new ArrayAdapter<>(requireContext(), android.R.layout.simple_spinner_item, ZegoConstants.googleTransLanguages);
        googleTranslateLanguageAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);

        sttProviderAdapter = new ArrayAdapter<String>(requireContext(), android.R.layout.simple_spinner_item, ZegoConstants.providerStrings);
        sttProviderAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        sttProviderSelector.setAdapter(sttProviderAdapter);
        sttProviderSelector.setSelection(0);
        sttProviderSelector.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                ZegoRTServiceProvider serviceProvider = ZegoConstants.providerList[position];
                if (serviceProvider == ZegoRTServiceProvider.Iflytek) {
                    // 科大讯飞
                } else if (serviceProvider == ZegoRTServiceProvider.Google){
                    // 谷歌
                    Toast.makeText(requireContext(), "谷歌在国内环境不稳定，切换到海外环境.", Toast.LENGTH_SHORT).show();
                }
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });

        translateProviderAdapter = new ArrayAdapter<String>(requireContext(), android.R.layout.simple_spinner_item, ZegoConstants.providerStrings);
        translateProviderAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        translateProviderSelector.setAdapter(translateProviderAdapter);
        translateProviderSelector.setSelection(0);
        translateProviderSelector.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                ZegoRTServiceProvider serviceProvider = ZegoConstants.providerList[position];
                if (serviceProvider == ZegoRTServiceProvider.Iflytek) {
                    // 科大讯飞
                    translateLanguageSelector.setAdapter(iFlyTransLanguagesAdapter);
                } else if (serviceProvider == ZegoRTServiceProvider.Google){
                    // 谷歌
                    Toast.makeText(requireContext(), "谷歌在国内环境不稳定，切换到海外环境.", Toast.LENGTH_SHORT).show();
                    translateLanguageSelector.setAdapter(googleTranslateLanguageAdapter);
                }

                translateLanguageSelector.setSelection(0);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });

        translateLanguageSelector.setAdapter(iFlyTransLanguagesAdapter);
        translateLanguageSelector.setSelection(0);

        startRTFileButton.setOnClickListener(v -> {
            if (isStartRecognizeFile) {
                isStartRecognizeFile = false;
            } else {
                isStartRecognizeFile = true;
                readFile();
            }
        });
        return containerView;
    }

    private void readFile() {
        ZegoConstants.SpeechLanguage speechLanguage = pcmFileList[(int)pcmFileSelector.getSelectedItemId()];
        int rawFileID = pcmFileIDList[(int)pcmFileSelector.getSelectedItemId()];

        startRTFileButton.setText("停止文件语音识别");
        zegoRealtimeTranslation.setRealtimeTranslationHandler(realtimeTranslationHandler);
        zegoRealtimeTranslation.setRealtimeTranslationHandler(realtimeTranslationHandler);
        zegoRealtimeTranslation.enableTranslate(enableTranslateCheckBox.isChecked());
        zegoRealtimeTranslation.enableInterimTranslation(enableIntermediaTranslateCheckBox.isChecked());
        zegoRealtimeTranslation.setSpeechToTextService(ZegoConstants.providerList[(int) sttProviderSelector.getSelectedItemId()]);
        zegoRealtimeTranslation.setTextTranslationService(ZegoConstants.providerList[(int) translateProviderSelector.getSelectedItemId()]);
        zegoRealtimeTranslation.setSourceSpeechLanguage(speechLanguage.speechLanguage);
        ZegoConstants.TranslationLanguage translationLanguage = (ZegoConstants.TranslationLanguage)translateLanguageSelector.getSelectedItem();
        zegoRealtimeTranslation.setTargetTranslationLanguage(translationLanguage.translationLanguage);
        zegoRealtimeTranslation.startRealtimeTranslation();

        sttProviderSelector.setEnabled(false);
        translateProviderSelector.setEnabled(false);
        translateLanguageSelector.setEnabled(false);
        new Thread(() -> {
            AudioTrack audioTrack = new AudioTrack(AudioManager.STREAM_MUSIC, 16000, AudioFormat.CHANNEL_OUT_MONO,
                    AudioFormat.ENCODING_PCM_16BIT, 1280, AudioTrack.MODE_STREAM);
            int frameSize = 1280;
            int interval = 40;
            audioTrack.play();
            try (InputStream fs = requireContext().getResources().openRawResource(rawFileID)) {
                byte[] buffer = new byte[frameSize];
                while (isStartRecognizeFile) {
                    int len = fs.read(buffer);
                    if (len == -1) {
                        break;
                    }
                    audioTrack.write(buffer, 0, len);
                    zegoRealtimeTranslation.sendSpeechPCM(buffer, len);
                    Thread.sleep(interval); //模拟音频采样延时
                }
                audioTrack.stop();
                audioTrack.release();
                zegoRealtimeTranslation.stopRealtimeTranslation();
            } catch (InterruptedException ie) {
                Log.e(TAG, ie.getMessage());
                Thread.currentThread().interrupt();
            } catch (IOException ioe) {
                Log.e(TAG, ioe.getMessage());
            } finally {
                handler.post(() -> {
                    resetButton();
                });
                isStartRecognizeFile = false;
            }
        }).start();
    }

    public void resetButton() {
        startRTFileButton.setText("开始文件语音识别");
        sttProviderSelector.setEnabled(true);
        translateProviderSelector.setEnabled(true);
        translateLanguageSelector.setEnabled(true);
    }

    private IZegoRealtimeTranslationHandler realtimeTranslationHandler = new IZegoRealtimeTranslationHandler() {
        @Override
        public void onRecognizingText(String sttText) {
            if (!TextUtils.isEmpty(sttText)) {
                speechToTextAdapter.replaceLastText(sttText);
            }
        }

        @Override
        public void onRecognizingTranslatedText(String translateText) {
            if (!TextUtils.isEmpty(translateText)) {
                translateTextAdapter.replaceLastText(translateText);
            }
        }

        @Override
        public void onRecognizedText(String sttText) {
            if (!TextUtils.isEmpty(sttText)) {
                speechToTextAdapter.replaceLastText(sttText);
                speechToTextAdapter.addText(" ");
            }
        }

        @Override
        public void onRecognizedTranslatedText(String translateText) {
            if (!TextUtils.isEmpty(translateText)) {
                translateTextAdapter.replaceLastText(translateText);
                translateTextAdapter.addText(" ");
            }
        }

        @Override
        public void onError(int error) {
            Log.e(TAG, "ZegoRealtimeTranslation onError:" + error);
            if (Looper.myLooper() == Looper.getMainLooper()) {
                AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(requireActivity());
                dialogBuilder.setTitle("提示").setMessage("ZegoRealtimeTranslation onError:" + ZegoConstants.errorMap.get(error)).setPositiveButton("确定", (dialog, which) -> {
                });
                dialogBuilder.setOnCancelListener(dialog -> requireActivity().finish());
                dialogBuilder.show();
//                        Toast.makeText(LiveActivity.this, "ZegoRealtimeTranslation onError:" + ZegoConstants.errorMap.get(error), Toast.LENGTH_LONG).show();
            } else {
                handler.post(() -> {
                    AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(requireActivity());
                    dialogBuilder.setTitle("提示").setMessage("ZegoRealtimeTranslation onError:" + ZegoConstants.errorMap.get(error)).setPositiveButton("确定", (dialog, which) -> {
                    });
                    dialogBuilder.setOnCancelListener(dialog -> requireActivity().finish());
                    dialogBuilder.show();
//                            Toast.makeText(LiveActivity.this, "ZegoRealtimeTranslation onError:" + ZegoConstants.errorMap.get(error), Toast.LENGTH_LONG).show();
                });
            }

            resetButton();
        }
    };

    @Override
    public String toString() {
        return "语音文件";
    }
}