package im.zego.realtimetranslationdemo;

import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.TextureView;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.GridLayout;
import android.widget.TextView;
import android.widget.Toast;

import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentPagerAdapter;
import androidx.lifecycle.ViewModelProvider;
import androidx.viewpager.widget.ViewPager;

import com.google.android.material.tabs.TabLayout;
import com.google.gson.Gson;

import java.nio.ByteBuffer;
import java.util.ArrayList;

import im.zego.realtimetranslation.ZegoRealtimeTranslation;
import im.zego.realtimetranslation.ZegoRealtimeTranslationError;

public class LiveActivity extends AppCompatActivity {
    private static final String TAG = "LiveActivity";

    private LiveActivityViewModel viewModel;
    private TextView roomNameTv;
    private ViewGroup streamLayout;
    private ViewPager tabViewPager;
    private ArrayList<Fragment> fragmentList = new ArrayList<>();
    private SttFragment sttFragment;
    private TranslateFragment translateFragment;
    private AudioFileFragment audioFileFragment;

    private TextureView hostTextureView;
    private Gson gson = new Gson();
    private ArrayList<ViewGroup> streamViewList = new ArrayList<>();
    private RTCEngineManager rtcManager = RTCEngineManager.getInstance();
    private Handler handler = new Handler(Looper.getMainLooper());
    private ZegoRealtimeTranslation zegoRealtimeTranslation = ZegoRealtimeTranslation.getInstance();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Log.i(TAG, "onCreate");
//        ImmersionBar.with(LiveActivity.this).hideBar(BarHide.FLAG_HIDE_BAR).init();
        getSupportActionBar().hide();
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);
        setContentView(R.layout.activity_live);
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        viewModel = new ViewModelProvider(this).get(LiveActivityViewModel.class);

        init();
        initView();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        Log.i(TAG, "onDestroy");
        fragmentList.clear();
        streamViewList.clear();
        getWindow().clearFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        rtcManager.stopPublishStream();
        rtcManager.removeRTCEventListener(rtcEventListener);
        rtcManager.logoutRoom();
        zegoRealtimeTranslation.unInit();
    }

    private void init() {
        Intent intent = getIntent();
        viewModel.roomID = intent.getStringExtra(ZegoConstants.ROOM_ID_PARAMS);
        viewModel.roomName = intent.getStringExtra(ZegoConstants.ROOM_NAME_PARAMS);
        viewModel.userID = intent.getStringExtra(ZegoConstants.USERID_PARAMS);
        viewModel.userName = intent.getStringExtra(ZegoConstants.USERNAME_PARAMS);
        viewModel.isHost = intent.getBooleanExtra(ZegoConstants.IS_HOST_PARAMS, false);
        rtcManager.addRTCEventListener(rtcEventListener);
        try {
            int ret = zegoRealtimeTranslation.init(KeyCenter.license);
            if (ret != ZegoRealtimeTranslationError.SUCCESS) {
                Toast.makeText(this, "ZegoRealtimeTranslation init failed:" + ZegoConstants.errorMap.get(ret), Toast.LENGTH_LONG).show();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void initView() {
        roomNameTv = findViewById(R.id.room_name_tv);
        streamLayout = findViewById(R.id.stream_layout);
        findViewById(R.id.back_btn).setOnClickListener(v -> {
            finish();
        });
        TabLayout tabLayout = findViewById(R.id.tab_layout);
        tabViewPager = findViewById(R.id.tab_viewpager);
        tabLayout.addOnTabSelectedListener(new TabLayout.OnTabSelectedListener() {
            @Override
            public void onTabSelected(TabLayout.Tab tab) {
                tabViewPager.setCurrentItem(tab.getPosition(), true);
            }

            @Override
            public void onTabUnselected(TabLayout.Tab tab) {

            }

            @Override
            public void onTabReselected(TabLayout.Tab tab) {

            }
        });
        sttFragment = SttFragment.newInstance();
        translateFragment = TranslateFragment.newInstance();
        audioFileFragment = AudioFileFragment.newInstance();
        fragmentList.add(sttFragment);
        fragmentList.add(translateFragment);
        fragmentList.add(audioFileFragment);

        tabLayout.setupWithViewPager(tabViewPager);

        tabViewPager.setAdapter(new FragmentPagerAdapter(getSupportFragmentManager()) {
            //前面两个方法照常写
            @Override
            public int getCount() {
                return fragmentList.size();
            }

            @Override
            public Fragment getItem(int position) {
                return fragmentList.get(position);
            }

            @Override
            public CharSequence getPageTitle(int position) {
                return getItem(position).toString();
            }
        });

        roomNameTv.setText(viewModel.roomName);
        ArrayList<RTCEngineManager.StreamInfo> streamList = rtcManager.getStreamInfoList();

        if (viewModel.isHost) {
            // 先判断是否已经有达到上限的主播在房间里了
            if (rtcManager.getStreamInfoList().size() >= ZegoConstants.MAX_HOST_NUMBER) {
                runOnUiThread(() -> {
                    AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(LiveActivity.this);
                    dialogBuilder.setTitle("提示").setMessage("主播人数已满，请换个房间。").setPositiveButton("确定", (dialog, which) -> {
                        finish();
                    });
                    dialogBuilder.setOnCancelListener(dialog -> finish());
                    dialogBuilder.show();
                });
                return;
            }

            ViewGroup mainStreamView = (ViewGroup) LayoutInflater.from(this).inflate(R.layout.pk_stream_view, null);
            hostTextureView = mainStreamView.findViewById(R.id.stream_texture_view);
            TextView textView = mainStreamView.findViewById(R.id.room_info);
            textView.setText(viewModel.userName);
            mainStreamView.findViewById(R.id.close_pk_view_btn).setVisibility(View.GONE);
            GridLayout.LayoutParams params = new GridLayout.LayoutParams();
            params.width = 0;
            params.height = 0;
            params.rowSpec = GridLayout.spec(0, 1, 1);
            params.columnSpec = GridLayout.spec(0, 1, 1);
            streamLayout.addView(mainStreamView, params);
            streamViewList.add(mainStreamView);

            rtcManager.startPublishStream();
            mainStreamView.setTag(rtcManager.publishStreamID);
            rtcManager.startPreview(hostTextureView);
        }

        if (streamList.size() > 0) {
            handleStreamsAdded(streamList);
        }

        if (!rtcManager.isInRoom()) {
            AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(LiveActivity.this);
            dialogBuilder.setTitle("提示").setMessage("你已经离线。").setPositiveButton("确定", (dialog, which) -> {
                finish();
            });
            dialogBuilder.setOnCancelListener(dialog -> finish());
            dialogBuilder.show();
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
    }

    @Override
    public boolean onKeyUp(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            finish();
        }
        return super.onKeyUp(keyCode, event);
    }

    private RTCEngineManager.IRTCEventListener rtcEventListener = new RTCEngineManager.IRTCEventListener() {
        @Override
        public void onStreamAdd(ArrayList<RTCEngineManager.StreamInfo> streamInfoList) {
            handleStreamsAdded(streamInfoList);
        }

        @Override
        public void onStreamRemove(ArrayList<RTCEngineManager.StreamInfo> streamInfoList) {
            for (RTCEngineManager.StreamInfo streamInfo : streamInfoList) {
                if (streamInfo.streamID.startsWith(RTCEngineManager.STREAM_PREFIX)) {
                    for (ViewGroup viewGroup: streamViewList) {
                        String tag = (String)viewGroup.getTag();
                        if (tag.equals(streamInfo.streamID)) {
                            streamViewList.remove(viewGroup);
                            rtcManager.stopPlayingStream(streamInfo.streamID);
                            refreshHostView();
                            break;
                        }
                    }
                }
            }
        }

        @Override
        public void onRoomStateChanged(RTCEngineManager.RoomStateChangeReason reason) {
            if (reason == RTCEngineManager.RoomStateChangeReason.KickOut) {
                runOnUiThread(() -> {
                    AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(LiveActivity.this);
                    dialogBuilder.setTitle("提示").setMessage("你已在其他设备登录。").setPositiveButton("确定", (dialog, which) -> {
                        finish();
                    });
                    dialogBuilder.setOnCancelListener(dialog -> finish());
                    dialogBuilder.show();
                });
            } else if (reason == RTCEngineManager.RoomStateChangeReason.Disconnect || reason == RTCEngineManager.RoomStateChangeReason.LoginFailed) {
                runOnUiThread(() -> {
                    AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(LiveActivity.this);
                    dialogBuilder.setTitle("提示").setMessage("你已经离线。").setPositiveButton("确定", (dialog, which) -> {
                        finish();
                    });
                    dialogBuilder.setOnCancelListener(dialog -> finish());
                    dialogBuilder.show();
                });
            }
        }
    };

    private void handleStreamsAdded(ArrayList<RTCEngineManager.StreamInfo> streamInfoList) {
        for (RTCEngineManager.StreamInfo streamInfo: streamInfoList) {
            if (streamInfo.streamID.startsWith(RTCEngineManager.STREAM_PREFIX)) {
                if (streamInfo.streamID.equals(rtcManager.publishStreamID)) {
                    continue;
                }
                ViewGroup streamView = (ViewGroup) LayoutInflater.from(LiveActivity.this).inflate(R.layout.pk_stream_view, null);
                TextureView newTextureView = streamView.findViewById(R.id.stream_texture_view);
                TextView textView = streamView.findViewById(R.id.room_info);
                textView.setText(streamInfo.userName);
                streamView.setTag(streamInfo.streamID);
                streamView.findViewById(R.id.close_pk_view_btn).setVisibility(View.GONE);
                streamViewList.add(streamView);
                rtcManager.startPlayingStream(streamInfo.streamID, newTextureView);
                refreshHostView();
            }
        }
    }
    private void refreshHostView() {
        int total = streamViewList.size();
        streamLayout.removeAllViews();

        for (int i = 0; i < total; i++) {
            ViewGroup streamView = streamViewList.get(i);
            String streamID = (String)streamView.getTag();
            TextureView textureView = streamView.findViewById(R.id.stream_texture_view);
            GridLayout.LayoutParams params = new GridLayout.LayoutParams();
            params.width = 0;
            params.height = 0;
            if (total < 5) {
                params.rowSpec = GridLayout.spec(i / 2, 1f);
                params.columnSpec = GridLayout.spec(i % 2, 1f);
            } else {
                params.rowSpec = GridLayout.spec(i / 3, 1f);
                params.columnSpec = GridLayout.spec(i % 3, 1f);
            }

            if (viewModel.isHost && i == 0) {
                // 自己的视频窗口使用预览
                rtcManager.startPreview(textureView);
            } else {
                rtcManager.startPlayingStream(streamID, textureView);
            }

            streamLayout.addView(streamView, params);
        }
    }

}