package im.zego.realtimetranslationdemo;

import android.Manifest;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.provider.Settings;
import android.text.TextUtils;
import android.util.Log;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Toast;

import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.content.ContextCompat;

import im.zego.realtimetranslationdemo.permissions.api.ZegoPermission;

public class MainActivity extends AppCompatActivity {
    private static final String TAG = "MainActivity";
    private RTCEngineManager rtcManager;
    private String roomID;
    private String randomNames[] = { "张三", "李四", "王五", "武则天", "秦始皇", "刘邦", "项羽", "女娲", "王昭君", "西施",
            "貂蝉", "杨玉环", "赵飞燕", "上官婉儿", "孙悟空", "唐僧", "猪八戒", "牛魔王", "盘古", "太上老君", "周瑜", "吴刚"
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Intent intent = new Intent(this, ForegroundService.class);
        startService(intent);
        setContentView(R.layout.activity_main);
        if (!ZegoPermission.hasAudioPermission(getApplication()) || !ZegoPermission.hasCameraPermission(getApplication())) {
            ZegoPermission.requestCameraAndMicPermissionInner(this);
        }

        rtcManager = RTCEngineManager.getInstance();
        rtcManager.initRtc(getApplication());

        findViewById(R.id.room1).setOnClickListener(v -> {
            onClickRoomBtn(1);
        });

        findViewById(R.id.room2).setOnClickListener(v -> {
            onClickRoomBtn(2);
        });
        findViewById(R.id.room3).setOnClickListener(v -> {
            onClickRoomBtn(3);
        });
        findViewById(R.id.room4).setOnClickListener(v -> {
            onClickRoomBtn(4);
        });
        findViewById(R.id.room5).setOnClickListener(v -> {
            onClickRoomBtn(5);
        });
        findViewById(R.id.room6).setOnClickListener(v -> {
            onClickRoomBtn(6);
        });
        findViewById(R.id.room7).setOnClickListener(v -> {
            onClickRoomBtn(7);
        });
        findViewById(R.id.room8).setOnClickListener(v -> {
            onClickRoomBtn(8);
        });
        findViewById(R.id.room9).setOnClickListener(v -> {
            onClickRoomBtn(9);
        });
        findViewById(R.id.room10).setOnClickListener(v -> {
            onClickRoomBtn(10);
        });
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        Intent intent = new Intent(this, ForegroundService.class);
        stopService(intent);
    }

    private void onClickRoomBtn(int index) {
        ViewGroup inputLayout = (ViewGroup) LayoutInflater.from(this).inflate(R.layout.select_role_dialog, null);
        EditText userNameEdit = inputLayout.findViewById(R.id.user_nickname_edit);
        Spinner roleSelector = inputLayout.findViewById(R.id.role_selector);
        Button enterRoomBtn = inputLayout.findViewById(R.id.enter_room_btn);
        int randomIndex = (int)(Math.random() * randomNames.length);
        userNameEdit.setText(randomNames[randomIndex]);
        String roleStrings[] = {"主播", "观众"};
        ArrayAdapter<String> sttProviderAdapter = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, roleStrings);
        sttProviderAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        roleSelector.setAdapter(sttProviderAdapter);
        roleSelector.setSelection(0);
        AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(this);
        dialogBuilder.setView(inputLayout);
        AlertDialog dialog = dialogBuilder.create();
        enterRoomBtn.setOnClickListener(v -> {
            int selectIndex = roleSelector.getSelectedItemPosition();
            String userName = userNameEdit.getText().toString();
            Boolean isHost = selectIndex != 1; // 2是观众
            if (TextUtils.isEmpty(userName)) {
                Toast.makeText(this, "用户名不能为空", Toast.LENGTH_LONG).show();
                return;
            }

            String userID = System.currentTimeMillis() + "";

            String newRoomID = index + "";

            if (!newRoomID.equals(roomID) || !rtcManager.isInRoom()) {
                rtcManager.logoutRoom();
                roomID = newRoomID;
                String finalUserID = userID;
                rtcManager.loginRoom(roomID, roomID, userID, userName, new IExecuteCallback<Object>()
                {
                    @Override
                    public void onResult(int errorCode, Object result) {
                        Log.i(TAG, "login $roomID success");
                        startLiveActivity(finalUserID, userName, index + "", "房间" + index, isHost);
                    }
                });
            } else {
                startLiveActivity(userID, userName, index + "", "房间" + index, isHost);
            }

            dialog.dismiss();
        });

        dialog.show();
    }

    private void startLiveActivity(String userID, String userName, String roomID, String roomName, Boolean isHost) {
        Intent intent = new Intent(this, LiveActivity.class);
        intent.putExtra(ZegoConstants.USERID_PARAMS, userID);
        intent.putExtra(ZegoConstants.USERNAME_PARAMS, userName);
        intent.putExtra(ZegoConstants.ROOM_ID_PARAMS, roomID);
        intent.putExtra(ZegoConstants.ROOM_NAME_PARAMS, roomName);
        intent.putExtra(ZegoConstants.IS_HOST_PARAMS, isHost);
        startActivity(intent);
    }

    @Override
    public boolean onKeyUp(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            rtcManager.unInitRtc(null);
            finish();
        }
        return super.onKeyUp(keyCode, event);
    }
}