package im.zego.realtimetranslationdemo;

import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.text.TextUtils;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.Spinner;
import android.widget.Toast;

import androidx.annotation.Keep;
import androidx.appcompat.app.AlertDialog;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.google.gson.Gson;
import com.google.gson.annotations.SerializedName;

import java.nio.ByteBuffer;

import im.zego.realtimetranslation.IZegoRealtimeTranslationHandler;
import im.zego.realtimetranslation.ZegoRealtimeTranslation;
import im.zego.realtimetranslation.ZegoRealtimeTranslationError;
import im.zego.realtimetranslation.ZegoRTServiceProvider;

/**
 * A simple {@link Fragment} subclass.
 * Use the {@link SttFragment#newInstance} factory method to
 * create an instance of this fragment.
 */
public class SttFragment extends Fragment {

    private static final String TAG = "SttFragment";
    private Button switchSpeechToTextBtn;
    private boolean isOpenStt = false;
    private RecyclerView speechToTextRecyclerView;
    private RecyclerView translateTextRecyclerView;
    public TextAdapter speechToTextAdapter = new TextAdapter();
    public TextAdapter translateTextAdapter = new TextAdapter();
    private CheckBox enableTranslateCheckBox;
    private CheckBox enableIntermediaTranslateCheckBox;
    private ViewGroup hostLayout;
    private Spinner sttProviderSelector;
    private ArrayAdapter<String> sttProviderAdapter;

    private Spinner translateProviderSelector;
    private ArrayAdapter<String> translateProviderAdapter;

    private Spinner fromLanguageSelector;
    private ArrayAdapter<ZegoConstants.SpeechLanguage> iFlySttLanguagesAdapter;
    private ArrayAdapter<ZegoConstants.SpeechLanguage> googleSttLanguageAdapter;
    private ArrayAdapter<ZegoConstants.SpeechLanguage> iLiveDataSttLanguageAdapter;

    private Spinner translateLanguageSelector;
    private ArrayAdapter<ZegoConstants.TranslationLanguage> googleTranslateLanguageAdapter;
    private ArrayAdapter<ZegoConstants.TranslationLanguage> iFlyTransLanguagesAdapter;
    private ArrayAdapter<ZegoConstants.TranslationLanguage> iLiveDataTransLanguagesAdapter;
    private LiveActivityViewModel viewModel;
    private ZegoRealtimeTranslation zegoRealtimeTranslation = ZegoRealtimeTranslation.getInstance();
    private RTCEngineManager rtcEngineManager = RTCEngineManager.getInstance();
    private Gson gson = new Gson();
    private Handler handler = new Handler(Looper.getMainLooper());
    private long lastClickTime = 0;
    public SttFragment() {
    }

    /**
     * Use this factory method to create a new instance of
     * this fragment using the provided parameters.
     *
     * @return A new instance of fragment SttFragment.
     */
    public static SttFragment newInstance() {
        SttFragment fragment = new SttFragment();
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        viewModel = new ViewModelProvider(requireActivity()).get(LiveActivityViewModel.class);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View containerView = inflater.inflate(R.layout.fragment_stt, container, false);
        switchSpeechToTextBtn = containerView.findViewById(R.id.switch_stt_btn);
        speechToTextRecyclerView = containerView.findViewById(R.id.speech_to_text_recycler_view);
        translateTextRecyclerView = containerView.findViewById(R.id.translate_text_recycler_view);
        sttProviderSelector = containerView.findViewById(R.id.speech_to_text_sdk);
        translateProviderSelector = containerView.findViewById(R.id.translate_sdk);
        fromLanguageSelector = containerView.findViewById(R.id.from_language);
        translateLanguageSelector = containerView.findViewById(R.id.to_language);
        enableTranslateCheckBox = containerView.findViewById(R.id.checkbox_enable_translate);
        enableIntermediaTranslateCheckBox = containerView.findViewById(R.id.checkbox_enable_intermedia_translate);
        containerView.findViewById(R.id.clear_screen).setOnClickListener(v -> {
            speechToTextAdapter.clear();
            translateTextAdapter.clear();
        });

        rtcEngineManager.addRTCEventListener(rtcEventListener);

        hostLayout = containerView.findViewById(R.id.host_layout);
        enableTranslateCheckBox.setOnCheckedChangeListener((buttonView, isChecked) -> {
            zegoRealtimeTranslation.enableTranslate(isChecked);
            if (isChecked) {
                enableIntermediaTranslateCheckBox.setVisibility(View.VISIBLE);
            } else {
                enableIntermediaTranslateCheckBox.setVisibility(View.GONE);
            }
        });
        enableIntermediaTranslateCheckBox.setOnCheckedChangeListener((buttonView, isChecked) -> {
            zegoRealtimeTranslation.enableInterimTranslation(isChecked);
        });

        LinearLayoutManager linearLayoutManager1 = new LinearLayoutManager(requireContext());
        linearLayoutManager1.setStackFromEnd(true);
        speechToTextRecyclerView.setLayoutManager(linearLayoutManager1);
        speechToTextRecyclerView.setAdapter(speechToTextAdapter);
        speechToTextAdapter.setRecyclerView(speechToTextRecyclerView);

        LinearLayoutManager linearLayoutManager2 = new LinearLayoutManager(requireContext());
        linearLayoutManager2.setStackFromEnd(true);
        translateTextRecyclerView.setLayoutManager(linearLayoutManager2);
        translateTextRecyclerView.setAdapter(translateTextAdapter);
        translateTextAdapter.setRecyclerView(translateTextRecyclerView);

        iFlySttLanguagesAdapter = new ArrayAdapter<>(requireContext(), android.R.layout.simple_spinner_item, ZegoConstants.iflySttLanguages);
        iFlySttLanguagesAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        iFlyTransLanguagesAdapter = new ArrayAdapter<>(requireContext(), android.R.layout.simple_spinner_item, ZegoConstants.iflyTransLanguages);
        iFlyTransLanguagesAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        googleSttLanguageAdapter = new ArrayAdapter<>(requireContext(), android.R.layout.simple_spinner_item, ZegoConstants.googleSttLanguages);
        googleSttLanguageAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        googleTranslateLanguageAdapter = new ArrayAdapter<>(requireContext(), android.R.layout.simple_spinner_item, ZegoConstants.googleTransLanguages);
        googleTranslateLanguageAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        iLiveDataSttLanguageAdapter = new ArrayAdapter<>(requireContext(), android.R.layout.simple_spinner_item, ZegoConstants.liveDataSttLanguages);
        iLiveDataSttLanguageAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        iLiveDataTransLanguagesAdapter = new ArrayAdapter<>(requireContext(), android.R.layout.simple_spinner_item, ZegoConstants.iLiveDataTransLanguages);
        iLiveDataTransLanguagesAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);

        sttProviderAdapter = new ArrayAdapter<String>(requireContext(), android.R.layout.simple_spinner_item, ZegoConstants.providerStrings);
        sttProviderAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        sttProviderSelector.setAdapter(sttProviderAdapter);
        sttProviderSelector.setSelection(0);
        sttProviderSelector.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                ZegoRTServiceProvider serviceProvider = ZegoConstants.providerList[position];
                if (serviceProvider == ZegoRTServiceProvider.Iflytek) {
                    // 科大讯飞
                    fromLanguageSelector.setAdapter(iFlySttLanguagesAdapter);
                } else if (serviceProvider == ZegoRTServiceProvider.Google){
                    // 谷歌
                    fromLanguageSelector.setAdapter(googleSttLanguageAdapter);
                    Toast.makeText(requireContext(), "谷歌在国内环境不稳定，切换到海外环境.", Toast.LENGTH_SHORT).show();
                } else if (serviceProvider == ZegoRTServiceProvider.LiveData) {
                    // 云上曲率
                    fromLanguageSelector.setAdapter(iLiveDataSttLanguageAdapter);
                }

                fromLanguageSelector.setSelection(0);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });

        translateProviderAdapter = new ArrayAdapter<String>(requireContext(), android.R.layout.simple_spinner_item, ZegoConstants.providerStrings);
        translateProviderAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        translateProviderSelector.setAdapter(translateProviderAdapter);
        translateProviderSelector.setSelection(0);
        translateProviderSelector.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                ZegoRTServiceProvider serviceProvider = ZegoConstants.providerList[position];
                if (serviceProvider == ZegoRTServiceProvider.Iflytek) {
                    // 科大讯飞
                    translateLanguageSelector.setAdapter(iFlyTransLanguagesAdapter);
                } else if (serviceProvider == ZegoRTServiceProvider.Google) {
                    // 谷歌
                    Toast.makeText(requireContext(), "谷歌在国内环境不稳定，切换到海外环境.", Toast.LENGTH_SHORT).show();
                    translateLanguageSelector.setAdapter(googleTranslateLanguageAdapter);
                } else if (serviceProvider == ZegoRTServiceProvider.LiveData) {
                    translateLanguageSelector.setAdapter(iLiveDataTransLanguagesAdapter);
                }

                translateLanguageSelector.setSelection(1);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });

        fromLanguageSelector.setAdapter(iFlySttLanguagesAdapter);
        fromLanguageSelector.setSelection(0);

        translateLanguageSelector.setAdapter(iFlyTransLanguagesAdapter);
        translateLanguageSelector.setSelection(1);

        if (viewModel.isHost) {
            hostLayout.setVisibility(View.VISIBLE);
            switchSpeechToTextBtn.setVisibility(View.VISIBLE);
        } else {
            hostLayout.setVisibility(View.GONE);
            switchSpeechToTextBtn.setVisibility(View.GONE);
        }

        switchSpeechToTextBtn.setOnClickListener(v -> {
            if (System.currentTimeMillis() - lastClickTime < 300) {
                return;
            }

            lastClickTime = System.currentTimeMillis();
            if (isOpenStt) {
                zegoRealtimeTranslation.stopRealtimeTranslation();
                isOpenStt = false;
                switchSpeechToTextBtn.setText("开启实时语音识别");
                sttProviderSelector.setEnabled(true);
                translateProviderSelector.setEnabled(true);
                fromLanguageSelector.setEnabled(true);
                translateLanguageSelector.setEnabled(true);
//                handler.removeCallbacks(timeOutRunnable);
            } else {
//                Toast.makeText(requireContext(), "实时转译体验时长为2分钟", Toast.LENGTH_SHORT).show();

                zegoRealtimeTranslation.setRealtimeTranslationHandler(realtimeTranslationHandler);
                zegoRealtimeTranslation.enableTranslate(enableTranslateCheckBox.isChecked());
                zegoRealtimeTranslation.enableInterimTranslation(enableIntermediaTranslateCheckBox.isChecked());
                zegoRealtimeTranslation.setSpeechToTextService(ZegoConstants.providerList[(int) sttProviderSelector.getSelectedItemId()]);
                zegoRealtimeTranslation.setTextTranslationService(ZegoConstants.providerList[(int) translateProviderSelector.getSelectedItemId()]);
                ZegoConstants.SpeechLanguage speechLanguage = (ZegoConstants.SpeechLanguage)fromLanguageSelector.getSelectedItem();
                zegoRealtimeTranslation.setSourceSpeechLanguage(speechLanguage.speechLanguage);
                ZegoConstants.TranslationLanguage translationLanguage = (ZegoConstants.TranslationLanguage)translateLanguageSelector.getSelectedItem();
                zegoRealtimeTranslation.setTargetTranslationLanguage(translationLanguage.translationLanguage);

                int ret = zegoRealtimeTranslation.startRealtimeTranslation();
                if (ret == ZegoRealtimeTranslationError.ALREADY_STARTED) {
                    Toast.makeText(requireContext(), "already startRealtimeTranslation", Toast.LENGTH_LONG).show();
                } else  if (ret != ZegoRealtimeTranslationError.SUCCESS) {
                    Toast.makeText(requireContext(), "startRealtimeTranslation failed:" + ZegoConstants.errorMap.get(ret), Toast.LENGTH_LONG).show();
                    return;
                }
                isOpenStt = true;
                switchSpeechToTextBtn.setText("关闭实时语音识别");

                sttProviderSelector.setEnabled(false);
                translateProviderSelector.setEnabled(false);
                fromLanguageSelector.setEnabled(false);
                translateLanguageSelector.setEnabled(false);
//                handler.postDelayed(timeOutRunnable, 1000 * 120);
            }
        });

        return containerView;
    }

    public void resetButton() {
        isOpenStt = false;
        switchSpeechToTextBtn.setText("开启实时语音识别");
        sttProviderSelector.setEnabled(true);
        translateProviderSelector.setEnabled(true);
        fromLanguageSelector.setEnabled(true);
        translateLanguageSelector.setEnabled(true);
    }

    private IZegoRealtimeTranslationHandler realtimeTranslationHandler = new IZegoRealtimeTranslationHandler() {
        @Override
        public void onRecognizingText(String sttText) {
            RecognizeModel recognizeModel = new RecognizeModel();
            recognizeModel.userName = viewModel.userName;
            recognizeModel.text = sttText;
            if (!TextUtils.isEmpty(sttText)) {
                speechToTextAdapter.replaceLastText(recognizeModel.userName + ":" + sttText);
            }

            rtcEngineManager.sendRecognizingText(gson.toJson(recognizeModel));
        }

        @Override
        public void onRecognizingTranslatedText(String translateText) {
            RecognizeModel recognizeModel = new RecognizeModel();
            recognizeModel.userName = viewModel.userName;
            if (!TextUtils.isEmpty(translateText)) {
                recognizeModel.translateText = translateText;
                translateTextAdapter.replaceLastText(recognizeModel.userName + ":" + translateText);
            }

            rtcEngineManager.sendRecognizingText(gson.toJson(recognizeModel));
        }

        @Override
        public void onRecognizedText(String sttText) {
            RecognizeModel recognizeModel = new RecognizeModel();
            recognizeModel.userName = viewModel.userName;
            recognizeModel.text = sttText;
            if (!TextUtils.isEmpty(sttText)) {
                speechToTextAdapter.replaceLastText(recognizeModel.userName + ":" + sttText);
                speechToTextAdapter.addText(" ");
            }

            rtcEngineManager.sendRecognizedText(gson.toJson(recognizeModel));
        }

        @Override
        public void onRecognizedTranslatedText(String translateText) {
            RecognizeModel recognizeModel = new RecognizeModel();
            recognizeModel.userName = viewModel.userName;

            if (!TextUtils.isEmpty(translateText)) {
                recognizeModel.translateText = translateText;
                translateTextAdapter.replaceLastText(recognizeModel.userName + ":" + translateText);
                translateTextAdapter.addText(" ");
            }

            rtcEngineManager.sendRecognizedText(gson.toJson(recognizeModel));
        }

        @Override
        public void onError(int error) {
            Log.e(TAG, "ZegoRealtimeTranslation onError:" + error);
            if (Looper.myLooper() == Looper.getMainLooper()) {
                AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(requireActivity());
                dialogBuilder.setTitle("提示").setMessage("ZegoRealtimeTranslation onError:" + ZegoConstants.errorMap.get(error)).setPositiveButton("确定", (dialog, which) -> {
                });
                dialogBuilder.setOnCancelListener(dialog -> requireActivity().finish());
                dialogBuilder.show();
//                        Toast.makeText(LiveActivity.this, "ZegoRealtimeTranslation onError:" + ZegoConstants.errorMap.get(error), Toast.LENGTH_LONG).show();
            } else {
                handler.post(() -> {
                    AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(requireActivity());
                    dialogBuilder.setTitle("提示").setMessage("ZegoRealtimeTranslation onError:" + ZegoConstants.errorMap.get(error)).setPositiveButton("确定", (dialog, which) -> {
                    });
                    dialogBuilder.setOnCancelListener(dialog -> requireActivity().finish());
                    dialogBuilder.show();
//                            Toast.makeText(LiveActivity.this, "ZegoRealtimeTranslation onError:" + ZegoConstants.errorMap.get(error), Toast.LENGTH_LONG).show();
                });
            }

            resetButton();
        }
    };

    private RTCEngineManager.IRTCEventListener rtcEventListener = new RTCEngineManager.IRTCEventListener() {
        @Override
        public void onRecognizingText(String jsonStr) {
            RecognizeModel recognizeModel = gson.fromJson(jsonStr, RecognizeModel.class);
            if (!TextUtils.isEmpty(recognizeModel.translateText)) {
                String transText = recognizeModel.userName + ":" + recognizeModel.translateText;
                translateTextAdapter.replaceLastText(transText);
            }


            if (!TextUtils.isEmpty(recognizeModel.text)) {
                speechToTextAdapter.replaceLastText(recognizeModel.userName + ":" + recognizeModel.text);
            }
        }

        @Override
        public void onRecognizedText(String jsonStr) {
            RecognizeModel recognizeModel = gson.fromJson(jsonStr, RecognizeModel.class);
            if (!TextUtils.isEmpty(recognizeModel.translateText)) {
                String transText = recognizeModel.userName + ":" + recognizeModel.translateText;
                translateTextAdapter.replaceLastText(transText);
                translateTextAdapter.addText(" ");
            }

            if (!TextUtils.isEmpty(recognizeModel.text)) {
                speechToTextAdapter.replaceLastText(recognizeModel.userName + ":" + recognizeModel.text);
                speechToTextAdapter.addText(" ");
            }
        }

        @Override
        public void onRoomStateChanged(RTCEngineManager.RoomStateChangeReason reason) {
            zegoRealtimeTranslation.stopRealtimeTranslation();
        }

        @Override
        public void onCaptureAudioData(ByteBuffer byteBuffer, int dataLength, int value) {
            if (isOpenStt) {
                zegoRealtimeTranslation.sendSpeechPCM(byteBufferToByteArray(byteBuffer), dataLength);
            }
        }
    };

//    private Runnable timeOutRunnable = new Runnable() {
//        @Override
//        public void run() {
//            handler.post(new Runnable() {
//                @Override
//                public void run() {
//                    Toast.makeText(requireContext(), "实时转译体验时长已到期", Toast.LENGTH_SHORT).show();
//                }
//            });
//
//            zegoRealtimeTranslation.stopRealtimeTranslation();
//            requireActivity().finish();
//        }
//    };

    public static byte[] byteBufferToByteArray(ByteBuffer bytes) {
        int len = bytes.limit() - bytes.position();
        byte[] bytes1 = new byte[len];
        bytes.get(bytes1);
        return bytes1;
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        rtcEngineManager.removeRTCEventListener(rtcEventListener);
    }

    @Override
    public String toString() {
        return "实时传译";
    }

    @Keep
    public static class RecognizeModel{
        @SerializedName("username")
        public String userName;
        @SerializedName("text")
        public String text;
        @SerializedName("trans")
        public String translateText;
    }
}