package im.zego.realtimetranslationdemo;

import android.content.ClipboardManager;
import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import java.util.ArrayList;

public class TextAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder>{
    private ArrayList<String> dataList = new ArrayList<>();
    private RecyclerView recyclerView;
    private Handler handler = new Handler(Looper.getMainLooper());

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new RecyclerView.ViewHolder(new TextView(parent.getContext())){};
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, int position) {
        TextView itemView = (TextView)holder.itemView;
        itemView.setText(dataList.get(position));
        itemView.setOnLongClickListener(v -> {
            ClipboardManager cmb = (ClipboardManager)itemView.getContext().getSystemService(Context.CLIPBOARD_SERVICE);
            cmb.setText(getAllText());
            return false;
        });
    }

    @Override
    public int getItemCount() {
        return dataList.size();
    }

    public void setRecyclerView(RecyclerView recyclerView) {
        this.recyclerView = recyclerView;
    }

    public void clear() {
        handler.post(() -> {
            dataList.clear();
            notifyDataSetChanged();
        });
    }

    public void replaceLastText(String text) {
        handler.post(() -> {
            if (dataList.size() > 0) {
                dataList.remove(dataList.size() - 1);
            }

            dataList.add(text);
            notifyDataSetChanged();
        });
    }

    public void addText(String text) {
        handler.post(() -> {
            dataList.add(text);
            if (dataList.size() > 400) {
                dataList.remove(0);
            }
            notifyDataSetChanged();
            recyclerView.scrollToPosition(dataList.size() - 1);
//            handler.postDelayed(new Runnable() {
//                @Override
//                public void run() {
//                    recyclerView.scrollToPosition(dataList.size() - 1);
//                }
//            }, 100);
        });
    }

    public String getAllText() {
        StringBuilder sb = new StringBuilder();
        for (String text: dataList) {
            sb.append(text + "\n");
        }

        return sb.toString();
    }
}
