package im.zego.realtimetranslationdemo;

import android.os.Bundle;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;

import im.zego.realtimetranslation.ZegoRealtimeTranslation;
import im.zego.realtimetranslation.ZegoRealtimeTranslationError;
import im.zego.realtimetranslation.ZegoRTServiceProvider;

/**
 * A simple {@link Fragment} subclass.
 * Use the {@link TranslateFragment#newInstance} factory method to
 * create an instance of this fragment.
 */
public class TranslateFragment extends Fragment {

    private LiveActivityViewModel viewModel;

    private EditText inputEditText;
    private TextView translateTextView;
    private TextView elapseTimeTextView;

    private Spinner translateProviderSelector;
    private ArrayAdapter<String> translateProviderAdapter;

    private Spinner fromLanguageSelector;
    private ArrayAdapter<ZegoConstants.TranslationLanguage> googleTranslateLanguageAdapter;
    private ArrayAdapter<ZegoConstants.TranslationLanguage> iFlyTransLanguagesAdapter;
    private ArrayAdapter<ZegoConstants.TranslationLanguage> iLiveDataTransLanguagesAdapter;

    private Spinner translateLanguageSelector;
    private ZegoRealtimeTranslation zegoRealtimeTranslation = ZegoRealtimeTranslation.getInstance();
    public TranslateFragment() {
    }

    /**
     * Use this factory method to create a new instance of
     * this fragment using the provided parameters.
     *
     * @return A new instance of fragment TranslateFragment.
     */
    public static TranslateFragment newInstance() {
        TranslateFragment fragment = new TranslateFragment();
        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        viewModel = new ViewModelProvider(requireActivity()).get(LiveActivityViewModel.class);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View containerView = inflater.inflate(R.layout.fragment_translate, container, false);
        inputEditText = containerView.findViewById(R.id.input_text);
        translateTextView = containerView.findViewById(R.id.translate_textview);
        elapseTimeTextView = containerView.findViewById(R.id.elapse_time_tv);
        translateProviderSelector = containerView.findViewById(R.id.translate_sdk);
        fromLanguageSelector = containerView.findViewById(R.id.from_language);
        translateLanguageSelector = containerView.findViewById(R.id.to_language);

        translateProviderAdapter = new ArrayAdapter<String>(requireContext(), android.R.layout.simple_spinner_item, ZegoConstants.providerStrings);
        translateProviderAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        translateProviderSelector.setAdapter(translateProviderAdapter);
        translateProviderSelector.setSelection(0);

        translateProviderSelector.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                ZegoRTServiceProvider serviceProvider = ZegoConstants.providerList[position];
                if (serviceProvider == ZegoRTServiceProvider.Iflytek) {
                    // 科大讯飞
                    fromLanguageSelector.setAdapter(iFlyTransLanguagesAdapter);
                    fromLanguageSelector.setSelection(0);

                    translateLanguageSelector.setAdapter(iFlyTransLanguagesAdapter);
                    translateLanguageSelector.setSelection(1);
                } else if (serviceProvider == ZegoRTServiceProvider.Google){
                    // 谷歌
                    fromLanguageSelector.setAdapter(googleTranslateLanguageAdapter);
                    fromLanguageSelector.setSelection(0);

                    translateLanguageSelector.setAdapter(googleTranslateLanguageAdapter);
                    translateLanguageSelector.setSelection(1);
                } else if (serviceProvider == ZegoRTServiceProvider.LiveData) {
                    // 云上曲率
                    fromLanguageSelector.setAdapter(iLiveDataTransLanguagesAdapter);
                    fromLanguageSelector.setSelection(0);

                    translateLanguageSelector.setAdapter(iLiveDataTransLanguagesAdapter);
                    translateLanguageSelector.setSelection(1);
                }
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });

        iFlyTransLanguagesAdapter = new ArrayAdapter<>(requireContext(), android.R.layout.simple_spinner_item, ZegoConstants.iflyTransLanguages);
        iFlyTransLanguagesAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);

        googleTranslateLanguageAdapter = new ArrayAdapter<>(requireContext(), android.R.layout.simple_spinner_item, ZegoConstants.googleTransLanguages);
        googleTranslateLanguageAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);

        iLiveDataTransLanguagesAdapter = new ArrayAdapter<>(requireContext(), android.R.layout.simple_spinner_item, ZegoConstants.iLiveDataTransLanguages);
        iLiveDataTransLanguagesAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);

        fromLanguageSelector.setAdapter(iFlyTransLanguagesAdapter);
        fromLanguageSelector.setSelection(0);

        translateLanguageSelector.setAdapter(iFlyTransLanguagesAdapter);
        translateLanguageSelector.setSelection(1);

        containerView.findViewById(R.id.clear_screen).setOnClickListener(v -> {
            inputEditText.setText("");
            translateTextView.setText("");
        });
        containerView.findViewById(R.id.translate_btn).setOnClickListener(v -> {
            String inputText = inputEditText.getText().toString();
            if (!TextUtils.isEmpty(inputText)) {
                long startTime = System.currentTimeMillis();

                ZegoConstants.TranslationLanguage fromLanguage = (ZegoConstants.TranslationLanguage)fromLanguageSelector.getSelectedItem();
                ZegoConstants.TranslationLanguage toLanguage = (ZegoConstants.TranslationLanguage)translateLanguageSelector.getSelectedItem();
                zegoRealtimeTranslation.translate(inputText, ZegoConstants.providerList[(int) translateProviderSelector.getSelectedItemId()],
                        fromLanguage.translationLanguage,
                        toLanguage.translationLanguage,
                        (errorCode, resultText) -> {
                            if (errorCode != ZegoRealtimeTranslationError.SUCCESS) {
                                Toast.makeText(requireContext(), "Translation failed:" + ZegoConstants.errorMap.get(errorCode), Toast.LENGTH_LONG).show();
                                return;
                            }
                            long translationTime = System.currentTimeMillis() - startTime;
                            translateTextView.setText(resultText);
                            elapseTimeTextView.setText(translationTime + "ms");
                        });
            }
        });

        return containerView;
    }

    @Override
    public String toString() {
        return "文本翻译";
    }
}