package im.zego.realtimetranslationdemo.permissions.api;

import androidx.annotation.Size;
import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.LifecycleOwner;

import im.zego.realtimetranslationdemo.permissions.core.LivePermissions;
import im.zego.realtimetranslationdemo.permissions.core.PermissionResult;

/**
 * Created by rocket_wang on 2020/4/28.
 */
public class PermissionBuilder {
    private LifecycleOwner owner;
    private LivePermissions livePermissions;
    private PermissionListener mGrantPermissionListener;    // 允许权限
    private PermissionListener mDenyPermissionListener;     // 拒绝且不提醒
    private PermissionListener mLowApiDenyPermissionListener;   // 低版本(<6.0)对Deny的兼容
    private PermissionListener mRationalePermissionListener;    // 拒绝，下次申请依然还会弹框

    PermissionBuilder(AppCompatActivity activity) {
        this.owner = activity;
        livePermissions = new LivePermissions(activity);
    }

    PermissionBuilder(Fragment fragment) {
        this.owner = fragment;
        livePermissions = new LivePermissions(fragment);
    }

    public boolean permissionPermanentlyDenied(String deniedPermission) {
        return livePermissions.permissionPermanentlyDenied(deniedPermission);
    }

    // 允许
    public PermissionBuilder grant(PermissionListener listener) {
        this.mGrantPermissionListener = listener;
        return this;
    }

    // 拒绝且不提醒
    public PermissionBuilder deny(PermissionListener listener) {
        this.mDenyPermissionListener = listener;
        return this;
    }

    // 低版本(<6.0)对Deny的兼容
    public PermissionBuilder lowApiDeny(PermissionListener listener) {
        this.mLowApiDenyPermissionListener = listener;
        return this;
    }

    // 拒绝，下次申请依然还会弹框
    public PermissionBuilder rationale(PermissionListener listener) {
        this.mRationalePermissionListener = listener;
        return this;
    }

    public void request(@Size(min = 1) String... perms) {
        livePermissions.request(perms)
                .observe(owner, permissionResult -> {
                    if (permissionResult instanceof PermissionResult.AllGrant) {
                        String[] grantPerms = ((PermissionResult.AllGrant) permissionResult).getGrantPermissions();
                        if (grantPerms.length > 0 && mGrantPermissionListener != null) {
                            mGrantPermissionListener.onResult(grantPerms);
                        }
                    } else if (permissionResult instanceof PermissionResult.NotAllGrant) {
                        String[] grantPerms = ((PermissionResult.NotAllGrant) permissionResult).getGrantPermissions();
                        if (grantPerms.length > 0 && mGrantPermissionListener != null) {
                            mGrantPermissionListener.onResult(grantPerms);
                        }
                        String[] denyPerms = ((PermissionResult.NotAllGrant) permissionResult).getDenyPermissions();
                        if (denyPerms.length > 0 && mDenyPermissionListener != null) {
                            mDenyPermissionListener.onResult(denyPerms);
                        }
                        String[] rationalePerms = ((PermissionResult.NotAllGrant) permissionResult).getRationalePermissions();
                        if (rationalePerms.length > 0
                                && mRationalePermissionListener != null) {
                            mRationalePermissionListener.onResult(rationalePerms);
                        }
                    } else {
                        String[] denyPerms = ((PermissionResult.LowApiDeny)permissionResult).getDenyPermissions();
                        if (denyPerms.length > 0 && mLowApiDenyPermissionListener != null) {
                            mLowApiDenyPermissionListener.onResult(denyPerms);
                        }
                    }
                });
    }

    public interface PermissionListener {
        void onResult(String[] perms);
    }
}
