package im.zego.realtimetranslationdemo.permissions.api

import android.Manifest
import android.Manifest.permission
import android.app.Application
import android.content.Context
import androidx.annotation.Keep
import androidx.annotation.Size
import androidx.appcompat.app.AppCompatActivity
import androidx.fragment.app.Fragment
import im.zego.realtimetranslationdemo.permissions.core.LivePermissions
import im.zego.realtimetranslationdemo.IExecuteCallback
import java.util.*

/**
 * Created by rocket_wang on 2020/4/28.
 *
 * 注意！此方案不能在Activity中复写[android.app.Activity.onRequestPermissionsResult]
 *
 * @see LivePermissions
 *
 * @see im.zego.roomkit.permissions.core.LiveFragment.onRequestPermissionsResult
 */
@Keep
object ZegoPermission {
    const val TAG = "ZegoPermission"
    const val ERROR_START_SETTING_UI = -10001
    @JvmStatic
    fun hasPermissions(context: Context, @Size(min = 1) vararg perms: String): Boolean {
        return LivePermissions().hasPermissions(context, *perms)
    }

    @JvmStatic
    fun hasCameraPermission(application: Application): Boolean {
        return hasPermissions(application, permission.CAMERA)
    }

    @JvmStatic
    fun hasAudioPermission(application: Application): Boolean {
        return hasPermissions(application, permission.RECORD_AUDIO)
    }

    @JvmStatic
    fun with(activity: AppCompatActivity): PermissionBuilder {
        return PermissionBuilder(activity)
    }

    @JvmStatic
    fun with(fragment: Fragment): PermissionBuilder {
        return PermissionBuilder(fragment)
    }

    @JvmStatic
    fun requestCameraAndMicPermissionInner(activity: AppCompatActivity) {
        // 第一次请求权限才处理
        with(activity).grant { perms: Array<String?>? ->
                var forceEnableMic = false
                var forceEnableCamera = false
                if (perms != null) {
                    for (perm in perms) {
                        if (perm == Manifest.permission.CAMERA) {
                            forceEnableCamera = true
                        } else if (perm == Manifest.permission.RECORD_AUDIO) {
                            forceEnableMic = true
                        }
                    }
                }
            }
            .rationale { perms: Array<String?>? ->
                if (perms != null) {
                    for (perm in perms) {
                        if (perm == Manifest.permission.CAMERA) {
                        } else if (perm == Manifest.permission.RECORD_AUDIO) {
                        }
                    }
                }
            }
            .deny { perms: Array<String?>? ->
                if (perms != null) {
                    for (perm in perms) {
                        if (perm == Manifest.permission.CAMERA) {
                        } else if (perm == Manifest.permission.RECORD_AUDIO) {
                        }
                    }
                }
            }
            .lowApiDeny { perms: Array<String?>? ->
                if (perms != null) {
                    for (perm in perms) {
                        if (perm == Manifest.permission.CAMERA) {
                        } else if (perm == Manifest.permission.RECORD_AUDIO) {
                        }
                    }
                }
            }
            .request(Manifest.permission.CAMERA, Manifest.permission.RECORD_AUDIO)
    }

    @JvmStatic
    fun requestCameraPermissionInner(fragment: Fragment, requestCode: Int, callback: IExecuteCallback<String>) {
            with(fragment)
                    .grant { perms: Array<String?>? ->
                    }
                    .rationale { perms: Array<String?>? ->
                    }
                    .deny { perms: Array<String?>? ->
                    }
                    .lowApiDeny { perms: Array<String?>? ->
                        callback.onResult(ERROR_START_SETTING_UI, "")
                    }
                    .request(Manifest.permission.CAMERA)
    }

    @JvmStatic
    fun requestAudioPermissionInner(fragment: Fragment, requestCode: Int, callback: IExecuteCallback<String>) {
            with(fragment)
                    .grant { perms: Array<String?>? ->
                    }
                    .rationale { perms: Array<String?>? ->
                    }
                    .deny { perms: Array<String?>? ->
                    }
                    .lowApiDeny { perms: Array<String?>? ->
                        callback.onResult(ERROR_START_SETTING_UI, "")
                    }
                    .request(Manifest.permission.RECORD_AUDIO)
    }

    @JvmStatic
    fun requestAudioPermissionInner(activity: AppCompatActivity) {
        with(activity)
            .grant { perms: Array<String?>? ->
            }
            .rationale { perms: Array<String?>? ->
            }
            .deny { perms: Array<String?>? ->
            }
            .lowApiDeny { perms: Array<String?>? ->
            }
            .request(Manifest.permission.RECORD_AUDIO)
    }
}