package im.zego.realtimetranslationdemo.permissions.core

import android.content.pm.PackageManager
import android.os.Build
import android.os.Bundle
import androidx.fragment.app.Fragment
import androidx.lifecycle.MutableLiveData
import im.zego.realtimetranslationdemo.permissions.core.PermissionResult.AllGrant
import im.zego.realtimetranslationdemo.permissions.core.PermissionResult.LowApiDeny
import im.zego.realtimetranslationdemo.permissions.core.PermissionResult.NotAllGrant
import im.zego.realtimetranslationdemo.permissions.support.PermissionSupport

/**
 *
 */
internal class LiveFragment : Fragment() {

    lateinit var liveData :MutableLiveData<PermissionResult>

    companion object {
        private const val PERMISSIONS_REQUEST_CODE = 100
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        retainInstance = true
    }

    fun requestPermissions(permissions: Array<out String>) {
        liveData = MutableLiveData()
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.M) {
            if (PermissionSupport.isSupport()) {
                liveData.value = LowApiDeny(permissions as Array<String>)
            }
        } else {
            requestPermissions(permissions, PERMISSIONS_REQUEST_CODE)
        }
    }


    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<out String>,
        grantResults: IntArray
    ) {
        if (requestCode == PERMISSIONS_REQUEST_CODE) {
            val grantPermissions = ArrayList<String>()
            val denyPermission = ArrayList<String>()
            val rationalePermission = ArrayList<String>()
            for ((index, value) in grantResults.withIndex()) {
                if (value == PackageManager.PERMISSION_DENIED) {
                    if (shouldShowRequestPermissionRationale(permissions[index])) {
                        rationalePermission.add(permissions[index])
                    } else {
                        denyPermission.add(permissions[index])
                    }
                } else {
                    grantPermissions.add(permissions[index])
                }
            }
            if (grantPermissions.size == permissions.size && denyPermission.isEmpty() && rationalePermission.isEmpty()) {
                liveData.value = AllGrant(grantPermissions.toTypedArray())
            } else {
                liveData.value = NotAllGrant(grantPermissions.toTypedArray(),
                        rationalePermission.toTypedArray(),
                        denyPermission.toTypedArray())
            }

        }
    }

    fun shouldShowRationale(permission: String): Boolean {
        return shouldShowRequestPermissionRationale(permission)
    }
}