package im.zego.realtimetranslationdemo.permissions.support;

import android.hardware.Camera;
import android.media.AudioFormat;
import android.media.AudioRecord;
import android.media.MediaRecorder;

import java.lang.reflect.Field;

/**
 * Created by rocket_wang on 2020/4/23.
 */
public class PermissionSupportHelper {
    public static boolean isCameraCanUse() {
        boolean canUse = true;
        Camera mCamera = null;
        try {
            mCamera = Camera.open();
            // setParameters 是针对魅族MX5 做的。MX5 通过Camera.open() 拿到的Camera
            // 对象不为null
            Camera.Parameters mParameters = mCamera.getParameters();
            mCamera.setParameters(mParameters);
        } catch (Exception e) {
            canUse = false;
        }
        if (mCamera != null) {
            mCamera.release();
        }
        return canUse;
    }

    public static boolean isHasCameraPermission() {
        Field fieldPassword = null;
        try {
            Camera camera = Camera.open();
            fieldPassword = camera.getClass().getDeclaredField("mHasPermission");
            fieldPassword.setAccessible(true);
            Boolean result = (Boolean) fieldPassword.get(camera);
            return result;
        } catch (Exception e) {
            e.printStackTrace();
            return true;
        }
    }

    /**
     * 判断是是否有录音权限
     */
    public static boolean isHasRecordPermission() {
        // 音频获取源
        int audioSource = MediaRecorder.AudioSource.MIC;
        // 设置音频采样率，44100是目前的标准，但是某些设备仍然支持22050，16000，11025
        int sampleRateInHz = 44100;
        // 设置音频的录制的声道CHANNEL_IN_STEREO为双声道，CHANNEL_CONFIGURATION_MONO为单声道
        int channelConfig = AudioFormat.CHANNEL_IN_STEREO;
        // 音频数据格式:PCM 16位每个样本。保证设备支持。PCM 8位每个样本。不一定能得到设备支持。
        int audioFormat = AudioFormat.ENCODING_PCM_16BIT;
        // 缓冲区字节大小
        int bufferSizeInBytes = AudioRecord.getMinBufferSize(sampleRateInHz,
                channelConfig, audioFormat);
        AudioRecord audioRecord = null;
        boolean hasRecordPermission = true;
        try {
            audioRecord = new AudioRecord(audioSource, sampleRateInHz,
                    channelConfig, audioFormat, bufferSizeInBytes);
            // 开始录制音频
            // 防止某些手机崩溃，例如联想
            audioRecord.startRecording();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (audioRecord != null) {
                /**
                 * 根据开始录音判断是否有录音权限
                 */
                hasRecordPermission = (audioRecord.getRecordingState() == AudioRecord.RECORDSTATE_RECORDING);
                audioRecord.stop();
                audioRecord.release();
                audioRecord = null;
            }
        }
        return hasRecordPermission ;
    }

//    /**
//     * 判断sdk是否有麦克风权限
//     * 因为sdk有权限的话就持有了麦克风资源，导致我们通过平常的方法不能判断
//     * @return
//     */
//    public static boolean isSdkHasRecordPermission() {
//        CountDownLatch countDownLatch = new CountDownLatch(1);
//        ZegoDeviceManager.getInstance().setDeviceCallback((String deviceName, int errorCode) -> {
//            Logger.i(TAG, "isSdkHasRecordPermission() deviceName = " + deviceName + "， errorCode = " + errorCode);
//            countDownLatch.countDown();
//        });
//        ZegoDeviceManager.getInstance().enableMicroPhone(true);
//        try {
//            countDownLatch.await(500, TimeUnit.MILLISECONDS);
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//        //不为1说明error callback调用了，即sdk获取麦克风资源失败
//        return countDownLatch.getCount() == 1;
//    }
}
