package im.zego.realtimetranslationdemo.rtcproxy;

import android.app.Application;
import android.util.Log;
import android.view.TextureView;

import org.json.JSONObject;

import java.nio.ByteBuffer;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;

import im.zego.realtimetranslationdemo.IExecuteCallback;
import im.zego.realtimetranslationdemo.KeyCenter;
import im.zego.realtimetranslationdemo.RTCEngineManager;
import im.zego.realtimetranslationdemo.ZegoConstants;
import im.zego.zegoexpress.ZegoExpressEngine;
import im.zego.zegoexpress.callback.IZegoAudioDataHandler;
import im.zego.zegoexpress.callback.IZegoEventHandler;
import im.zego.zegoexpress.constants.ZegoAudioChannel;
import im.zego.zegoexpress.constants.ZegoAudioDataCallbackBitMask;
import im.zego.zegoexpress.constants.ZegoAudioSampleRate;
import im.zego.zegoexpress.constants.ZegoPublisherState;
import im.zego.zegoexpress.constants.ZegoRoomState;
import im.zego.zegoexpress.constants.ZegoRoomStateChangedReason;
import im.zego.zegoexpress.constants.ZegoScenario;
import im.zego.zegoexpress.constants.ZegoUpdateType;
import im.zego.zegoexpress.constants.ZegoVideoConfigPreset;
import im.zego.zegoexpress.constants.ZegoViewMode;
import im.zego.zegoexpress.entity.ZegoAudioFrameParam;
import im.zego.zegoexpress.entity.ZegoCanvas;
import im.zego.zegoexpress.entity.ZegoEngineProfile;
import im.zego.zegoexpress.entity.ZegoPlayStreamQuality;
import im.zego.zegoexpress.entity.ZegoPublishStreamQuality;
import im.zego.zegoexpress.entity.ZegoRoomConfig;
import im.zego.zegoexpress.entity.ZegoRoomExtraInfo;
import im.zego.zegoexpress.entity.ZegoStream;
import im.zego.zegoexpress.entity.ZegoUser;
import im.zego.zegoexpress.entity.ZegoVideoConfig;

public class ExpressProxy {
    private static final String TAG = "ExpressProxy";
    private boolean isInit = false;
    private Application application;
    private ZegoExpressEngine engine = null;
    private RTCEngineManager rtcEngineManager;

    public ExpressProxy(RTCEngineManager rtcEngineManager) {
        this.rtcEngineManager = rtcEngineManager;
    }

    public void init(Application application) {
        if (isInit) {
            return;
        }

        isInit = true;
        this.application = application;
        if (engine == null) {
            ZegoEngineProfile zegoEngineProfile = new ZegoEngineProfile();
            zegoEngineProfile.appID = KeyCenter.appID;
            zegoEngineProfile.appSign = KeyCenter.appSign;
            zegoEngineProfile.application = application;
            zegoEngineProfile.scenario = ZegoScenario.GENERAL;
            engine = ZegoExpressEngine.createEngine(zegoEngineProfile, zegoEventHandler);

            ZegoVideoConfig zegoVideoConfig = new ZegoVideoConfig(ZegoVideoConfigPreset.PRESET_270P);
            engine.setVideoConfig(zegoVideoConfig);

            // 设置音频的数据回调
            engine.setAudioDataHandler(audioDataHandler);
        }
    }

    public void unInit(IExecuteCallback<String> callback) {
        isInit = false;
        engine = null;
        ZegoExpressEngine.destroyEngine(() -> {
            if (callback != null) {
                callback.onResult(0, "");
            }
        });
    }

    private IZegoEventHandler zegoEventHandler = new IZegoEventHandler() {
        @Override
        public void onRoomStateUpdate(String roomID, ZegoRoomState zegoRoomState, int errorCode, JSONObject jsonObject) {
            Log.i(TAG, "onRoomStateUpdate:" + roomID + ", errorCode:" + errorCode + ", roomState:" + zegoRoomState.name() + ", jsonObject:" + jsonObject.toString());
        }

        @Override
        public void onRoomStateChanged(String roomID, ZegoRoomStateChangedReason zegoRoomStateChangedReason, int errorCode, JSONObject jsonObject) {
            Log.i(TAG, "onRoomStateChanged:" + roomID + ", errorCode:" + errorCode + ", zegoRoomStateChangedReason:" + zegoRoomStateChangedReason.name() + ", jsonObject:" + jsonObject.toString());
            if (zegoRoomStateChangedReason == ZegoRoomStateChangedReason.KICK_OUT) {
                rtcEngineManager.onRoomStateChanged(RTCEngineManager.RoomStateChangeReason.KickOut);
            } else if (zegoRoomStateChangedReason == ZegoRoomStateChangedReason.RECONNECT_FAILED) {
                rtcEngineManager.onRoomStateChanged(RTCEngineManager.RoomStateChangeReason.Disconnect);
            } else if (zegoRoomStateChangedReason == ZegoRoomStateChangedReason.LOGIN_FAILED) {
                rtcEngineManager.onRoomStateChanged(RTCEngineManager.RoomStateChangeReason.LoginFailed);
            } else if (zegoRoomStateChangedReason == ZegoRoomStateChangedReason.RECONNECTED) {
                rtcEngineManager.onRoomStateChanged(RTCEngineManager.RoomStateChangeReason.Reconnected);
            }
        }

        @Override
        public void onRoomStreamUpdate(String roomID, ZegoUpdateType zegoUpdateType, ArrayList<ZegoStream> arrayList, JSONObject jsonObject) {
            Log.i(TAG, "[login][onRoomStreamUpdate] roomID: " + roomID + ", zegoUpdateType:" + zegoUpdateType);
            ArrayList<RTCEngineManager.StreamInfo> streamInfoList = new ArrayList<>();
            for (ZegoStream stream: arrayList) {
                RTCEngineManager.StreamInfo streamInfo = new RTCEngineManager.StreamInfo(stream.user.userID, stream.user.userName, stream.streamID);
                streamInfoList.add(streamInfo);
            }
            if (zegoUpdateType == ZegoUpdateType.ADD) {
                rtcEngineManager.onStreamAdd(streamInfoList);
            } else {
                rtcEngineManager.onStreamRemove(streamInfoList);
            }
        }

        @Override
        public void onRoomStreamExtraInfoUpdate(String roomID, ArrayList<ZegoStream> arrayList) {
        }

        @Override
        public void onRoomExtraInfoUpdate(String roomID, ArrayList<ZegoRoomExtraInfo> arrayList) {
        }

        @Override
        public void onPlayerRecvSEI(String streamID, byte[] bytes) {
            rtcEngineManager.onReceiveSEI(streamID, bytes);
        }

        @Override
        public void onPublisherQualityUpdate(String streamID, ZegoPublishStreamQuality zegoPublishStreamQuality) {
            Log.i(TAG, "onPublisherQualityUpdate:" + streamID + ", " + zegoPublishStreamQuality.audioKBPS + ", " + zegoPublishStreamQuality.videoKBPS);
        }

        @Override
        public void onPlayerQualityUpdate(String streamID, ZegoPlayStreamQuality zegoPlayStreamQuality) {
            Log.i(TAG, "onPlayerQualityUpdate:" + streamID + ", " + zegoPlayStreamQuality.audioKBPS + ", " + zegoPlayStreamQuality.videoKBPS);
        }

        @Override
        public void onPublisherStateUpdate(String streamID, ZegoPublisherState zegoPublisherState, int errorCode, JSONObject jsonObject) {
            Log.i(TAG, "onPublisherStateUpdate:" + streamID + ", " + zegoPublisherState.toString() + ", errorCode:" + errorCode + ", jsonObject:" + jsonObject);
        }
    };

    public void login(String roomID, String roomName, String userID, String userName, IExecuteCallback<Object> callback) {
        ZegoUser zegoUser = new ZegoUser(userID, userName);
        ZegoRoomConfig zegoRoomConfig = new ZegoRoomConfig();
        zegoRoomConfig.isUserStatusNotify = true;
        engine.loginRoom(roomID, zegoUser, zegoRoomConfig, (errorCode, jsonObject) -> {
            Log.i(TAG, "[login][onLoginResult] errorCode: " + errorCode);
                if (callback != null) {
                    callback.onResult(errorCode, jsonObject);
                }
        });
    }

    public void logout() {
        if (engine != null) {
            engine.logoutRoom();
        }
    }

    public void startPublishStream(String streamID) {
        engine.startPublishingStream(streamID);
        ZegoAudioFrameParam audioFrameParam = new ZegoAudioFrameParam();
        audioFrameParam.sampleRate = ZegoAudioSampleRate.ZEGO_AUDIO_SAMPLE_RATE_16K;
        audioFrameParam.channel = ZegoAudioChannel.MONO;
        engine.startAudioDataObserver(ZegoAudioDataCallbackBitMask.CAPTURED.value(), audioFrameParam);
    }

    public void stopPublishStream() {
        if (engine != null) {
            engine.stopPublishingStream();
            engine.stopAudioDataObserver();
        }
    }

    public void startPlayingStream(String streamID, Object view) {
        if (view != null) {
            ZegoCanvas zegoCanvas = new ZegoCanvas(view);
            zegoCanvas.viewMode = ZegoViewMode.ASPECT_FILL;
            engine.startPlayingStream(streamID, zegoCanvas);
        } else {
            engine.startPlayingStream(streamID);
        }
    }

    public void stopPlayingStream(String streamID) {
        engine.stopPlayingStream(streamID);
    }

    public void startPreview(TextureView textureView) {
        ZegoCanvas zegoCanvas = new ZegoCanvas(textureView);
        zegoCanvas.viewMode = ZegoViewMode.ASPECT_FILL;
        engine.startPreview(zegoCanvas);
    }

    public void sendSEIData(String data) {
        engine.sendSEI(data.getBytes(StandardCharsets.UTF_8));
    }

    private IZegoAudioDataHandler audioDataHandler = new IZegoAudioDataHandler() {
        @Override
        public void onCapturedAudioData(ByteBuffer byteBuffer, int dataLength, ZegoAudioFrameParam zegoAudioFrameParam) {
            rtcEngineManager.onCaptureAudioData(byteBuffer, dataLength, zegoAudioFrameParam.sampleRate.value());
        }
    };
}
